@@ -1,5 +1,7 @@
-from typing import Optional, Union
+from datetime import datetime, timezone, timedelta
+from typing import Optional, Union, Tuple, Any
 from ast import literal_eval
+from enum import Enum, IntEnum
 
 
 def isfloat(val: Union[int, float, str]):
@@ -68,6 +70,9 @@ def split_fullname(fullname: str, default: str = '',
     :param suffix:      Custom suffixes to append to the default list
     :return:            tuple
     """
+    if not fullname:
+        return '', ''
+    
     if prefix and not isinstance(prefix, (str, list, tuple)):
         raise TypeError('`prefix` must be a list/str for multi/single values.')
 
@@ -141,4 +146,72 @@ def valid_str_only(item, allow_bool: bool = False) -> bool:
         return True
     if allow_bool and item:
         return True
-    return False
\ No newline at end of file
+    return False
+
+
+def setup_pagination(
+    *, items: int = 10, total: int, page: int = 1, sort: str, direction: str = 'desc',
+    asc: str = 'asc', dirflipper: str = '-', restartpage: bool = True, **_
+) -> Tuple[str, int, int]:
+    """
+    Generate the values to be used in pagination
+    :param items:           Number of rows to show
+    :param total:           Total count for all rows
+    :param page:            Page num
+    :param sort:            Column to sort from
+    :param direction:       asc or desc
+    :param asc:             String to identify asc
+    :param dirflipper:      String to identify a desc order
+    :param restartpage:     If the page exceeds the maximimum, this shows page one instead
+    :return:                tuple: orderby, offset, items
+    """
+    direction = '' if direction == asc else dirflipper
+    orderby = f'{direction}{sort}'
+    if restartpage and items * (page - 1) > total:
+        page = 1
+    offset = (page - 1) * items
+    return orderby, offset, items
+
+
+def offset_datetime(dt: datetime, offset: str) -> datetime:
+    """
+    Offsite the datetime instance to their local time
+    :param dt:      Datetime to convert.
+    :param offset:  Offset the dt by this amount. Accepts +08:00 or +8.
+    :return:        Modified datetime according to offset
+    """
+    offset = offset.split(':')
+    if len(offset) == 2:
+        hours, mins = offset
+        hours = int(hours.strip())
+        mins = hours < 0 and int(mins.strip()) * -1 or int(mins.strip())
+        mins += 60 * hours
+    else:
+        hours = int(offset[0].strip())
+        mins = hours * 60
+    
+    tz = timezone(timedelta(minutes=mins))
+    return dt.astimezone(tz)
+
+
+# TESTME: Untested
+def list_object_methods(obj: object):
+    """
+    List what methods are available for an objectn. Still untested.
+    :param obj: Any class object (I think)
+    :return:    Not sure yet
+    """
+    return [method_name for method_name in dir(obj) if callable(getattr(obj, method_name))]
+
+
+def reverse_choices(choices: Union[Enum, IntEnum], value: Any):
+    """
+    For use with choices. Just pass the value and it will return the name associated to it.
+    Good for reversing a db value (int) to its readable name attr
+    :param choices: Enum, IntEnum
+    :param value:   The value of the choice
+    :return:        str
+    """
+    for i in choices:               # noqa
+        if i.value == value:
+            return i.name
\ No newline at end of file
